package com.caimei365.user.service.impl;

import com.caimei365.user.mapper.BaseMapper;
import com.caimei365.user.mapper.RegisterMapper;
import com.caimei365.user.model.*;
import com.caimei365.user.service.RegisterService;
import com.caimei365.user.components.RedisService;
import com.caimei365.user.utils.AliyunSmsUtil;
import com.caimei365.user.utils.Md5Util;
import com.caimei365.user.utils.RequestUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ServerWebExchange;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/8
 */
@Slf4j
@Service
public class RegisterServiceImpl implements RegisterService {

    @Resource
    private RedisService redisService;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private RegisterMapper registerMapper;

    /**
     * 普通机构入驻(注册)
     *
     * @param source            注册来源: 0网站 1小程序
     * @param userName          用户名
     * @param bindMobile        企业绑定手机号
     * @param password          密码
     * @param passWordConfirm   用户确认密码
     * @param smsCode           短信验证码
     * @param isAgreed          是否同意勾选同意协议，1是，其他否
     * @param unionId           微信unionId
     * @param nickName          微信昵称
     * @param avatarUrl         微信头像
     * @param serverWebExchange ServerWebExchange
     * @return BaseUser
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJsonVo<ClubPo> clubRegister(Integer source, String userName, String bindMobile, String password, String passWordConfirm, String smsCode, Integer isAgreed, String unionId, String nickName, String avatarUrl, ServerWebExchange serverWebExchange) {
        // 打印IP
        String ip = RequestUtil.getIp(serverWebExchange);
        log.info("X-Forwarded-For:" + ip);
        // 参数校验
        if (StringUtils.isBlank(userName) || StringUtils.isBlank(bindMobile)
            || StringUtils.isBlank(password) || StringUtils.isBlank(passWordConfirm) ||
            StringUtils.isBlank(smsCode)) {
            return ResponseJsonVo.error("参数异常", null);
        }
        if (!password.equals(passWordConfirm)) {
            return ResponseJsonVo.error("输入的密码不一致", null);
        }
        String redisSmsCode = (String) redisService.get("code:" + bindMobile);
        if (redisSmsCode.equals(smsCode)) {
            return ResponseJsonVo.error("手机验证码错误", null);
        }
        if (1 != isAgreed) {
            return ResponseJsonVo.error("请勾选同意协议", null);
        }
        // 查找用户表是否存在
        Integer dbUserId = baseMapper.getUserIdByMobile(bindMobile);
        if (dbUserId > 0) {
            return ResponseJsonVo.error("该手机号已被使用", null);
        }
        // 查找运营人员表是否存在
        Integer dbOperationId = baseMapper.getOperationIdByMobile(bindMobile);
        if (dbOperationId > 0) {
            return ResponseJsonVo.error("您已是机构运营人员，无需再注册机构", null);
        }
        /*
            组装用户数据 user
         */
        UserPo user = new UserPo();
        // 设置日期时间格式
        Date date = new Date();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String current = dateFormat.format(date);
        // 注册时间
        user.setRegisterTime(current);
        // 注册IP
        user.setRegisterIp(ip);
        // 注册来源: 0网站 1小程序
        user.setSource(source);
        // 用户类型，供应商1，会员机构3,普通机构4
        user.setRegisterUserTypeId(4);
        // 组织名称
        user.setName(userName);
        user.setUserName(userName);
        // 绑定手机号
        user.setBindMobile(bindMobile);
        // 用户身份: 1协销 2会员机构 3供应商 4普通机构
        user.setUserIdentity(4);
        // 用户权限: 2会员机构 3供应商 4协销 5普通机构
        user.setUserPermission(5);
        // 设置机构上线
        user.setClubStatus(90);
        // 设置密码
        user.setPassword(Md5Util.md5(password));
        // 同意协议
        user.setAgreeFlag(isAgreed);
        // 用户状态，1正常，0冻结
        user.setValidFlag(1);
        // 协销Id，默认采美官方
        user.setServiceProviderId(1342);
        // 协销状态，已上线
        user.setServiceProviderStatus(90);
        /*
            保存数据库 user
         */
        int insertFlag = registerMapper.insertClubUser(user);
        if (insertFlag < 1) {
            throw new RuntimeException("插入数据库异常user:" + user.toString());
        }
        /*
            组装机构数据
         */
        ClubPo club = new ClubPo();
        // 机构名称
        club.setName(user.getUserName());
        // 机构简称
        club.setSName(user.getUserName());
        // 联系手机
        club.setContractMobile(user.getBindMobile());
        // 联系人
        club.setLinkMan(user.getUserName());
        // 用户ID
        club.setUserId(user.getUserId());
        // 协销ID(spID)
        club.setServiceProviderId(1342);
        // 注册时间
        club.setAddTime(current);
        // 状态设置上线
        club.setStatus(90);
        /*
            保存数据库 club
         */
        int insertClubFlag = registerMapper.insertClub(club);
        if (insertClubFlag < 1) {
            throw new RuntimeException("插入数据库异常club:" + club.toString());
        }
        // user更新clubId
        user.setClubId(club.getClubId());
        registerMapper.updateUserClubId(user.getUserId(), club.getClubId());
        // 注册成功短信
        boolean smsFlag = AliyunSmsUtil.sendSms(bindMobile, 7, "{name:"+ bindMobile +",password:"+ password +"}");
        if (!smsFlag) {
            // 短信发送失败重试一次
            AliyunSmsUtil.sendSms(bindMobile, 7, "{name:"+ bindMobile +",password:"+ password +"}");
        }
        log.info("注册普通机构成功,手机号>>>" + bindMobile);
        /*
            绑定微信，成为机构运营人员
         */
        if (StringUtils.isNotBlank(nickName)) {
            OperationPo operation = new OperationPo();
            // 用户ID
            operation.setUserId(user.getUserId());
            // 机构ID
            operation.setClubId(club.getClubId());
            // 微信昵称
            operation.setNickName(nickName);
            // 微信头像
            operation.setAvatarUrl(avatarUrl);
            // 用户类型，1:机构类型，2供应商类型
            operation.setUserType(1);
            // 运营人员手机号
            operation.setMobile(bindMobile);
            // 联系人
            operation.setLinkName(userName);
            // 运营人员状态：1未绑定，2已绑定
            operation.setStatus(2);
            // 微信unionId
            operation.setUnionId(unionId);
            Map<Object, Object> infoData = redisService.getEntries("wxInfo:applets:" + unionId);
            String openId = (String) infoData.get("openId");
            // 微信openId
            operation.setOpenId(openId);
            // 添加时间
            operation.setAddTime(date);
            // 更新时间
            operation.setUpdateTime(date);
            // 绑定时间
            operation.setBindTime(date);
            // delFlag: 0 有效  其它无效
            operation.setDelFlag(0);
            /*
                保存数据库 operation
             */
            int insertOperationFlag = registerMapper.insertOperation(operation);
            if (insertOperationFlag < 1) {
                log.info(operation.getUserId() + " 插入数据库异常operation:" + operation.toString());
            }
            log.info("注册普通机构,默认绑定为机构运营人员");
        }
        return ResponseJsonVo.success(club);
    }
}

